--[[
Plugin that handles flashing of the screen
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Flash")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local LSM = LibStub("LibSharedMedia-3.0")
local db
local flash = {}

local defaults = {
	profile = {
		Duration = 0.6,
		Pulses = 2,
		MaxAlpha = 0.2,
		Color = {1, 0.5, 0.60, 1},
		Texture = "Solid",
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Flash", defaults)
	db = self.db.profile
	
	self:InitFrame()
end

function plugin:PluginEnable()
	self:RegisterMessage("RW_Flash")
end

function plugin:PluginDisable()
end

local function update(self, elapsed)
	self.elapsed = self.elapsed + elapsed
	if self.elapsed > self.Duration then		
		plugin:ResetFrame()
	end
	local progress = self.elapsed % self.PulseTime
	if progress > self.HalfPulse then	-- When the pulse is halfway then it should start to fase again
		progress = self.PulseTime - progress   -- Fade down
	end
	self:SetAlpha(progress * self.Inverse) -- Multiply with inverse to get 1 when we are at half pulse
end

function plugin:InitFrame()
	local frame = CreateFrame("Frame", "RW_FlashFrame", UIParent)
	frame:SetAllPoints()	
	local t = frame:CreateTexture(nil, "BACKGROUND")
	t:SetAllPoints()
	t:SetBlendMode("ADD")
	frame.t = t
	frame:Hide()	
	flash = frame
	
	self:ResetFrame()
end

function plugin:ResetFrame()
	flash:SetScript("OnUpdate", nil)
	flash.elapsed = 0
	flash:Hide()
end

function plugin:RW_Flash(message, color, duration, pulses, texture)
	flash:SetScript("OnUpdate", update)
	flash.Duration = duration or db.Duration
	flash.Pulses = pulses or db.Pulses
	flash.t:SetTexture(LSM:Fetch("background", db.Texture))
	
	flash.PulseTime = flash.Duration / flash.Pulses
	flash.HalfPulse = flash.PulseTime / 2
	flash.Inverse = 1 / flash.HalfPulse
	
	local r, g, b
	if color then
		r, g, b = unpack(color)
	else
		r, g, b = unpack(db.Color)
	end
	flash.t:SetVertexColor(r, g, b, db.MaxAlpha)
	
	flash:SetAlpha(0)
	flash:Show()
end

-------------------------------------
-- Options

function plugin:GetOptions()
	self.category = "Alert plugins"

	local function get(info)
		if type(db[info[#info]]) == "table" then
			return unpack(db[info[#info]])
		else
			return db[info[#info]]
		end
	end
	
	local function set(info, value, g, b)
		if g then
			local r, g, b = value, g, b
			local c = db[info[#info]]
			c[1], c[2], c[3] = r, g, b
		else
			db[info[#info]] = value
		end
	end
	
	local options = {
		type = "group",
		name = "Screen Flash",
		set = set,
		get = get,
		args  = {
			Duration = {
				type = "range",
				name = "Duration",
				max = 5,
				min = 0.1,
				step = 0.1,
			},
			Pulses = {
				type = "range",
				name = "Pulses",
				max = 10,
				min = 1,
				step = 1,
			},
			MaxAlpha = {
				type = "range",
				name = "Transparancy",
				max = 1,
				min = 0.1,
				step = 0.1
			},
			Color = {
				type = "color",
				name = "Color",
			},
			Texture = {
				type = "select",
				name = "Texture",
				dialogControl = "LSM30_Background",
				values = AceGUIWidgetLSMlists.background,
			},
		}
	}
	return options
end